#!/bin/bash
# This script aims to simplify the manipulation of RAMdisks under Mac OS X.
#
# Version : 1.0
# Date : 2009-12-26
# Author : Matthieu Sarter
# URL : http://matthieu.sarter.fr/autres-realisations/ramdisk-sh
#
function usage() {
  echo "Usage: RAMdisk.sh [-c <size> [<label>]|-d {all|<id>|<label>}]"
  echo "-c <size> [<label>]	Creates a RAMdisk of <size> MB named <label>."
  echo "-d all			Deletes all RAMdisks."
  echo "-d <id>			Deletes the RAMdisk /dev/disk<id>."
  echo "-d <label>		Deletes the RAMdisk named <label>."
  exit 1
}

if [ "$2" = "" -o "$1" != "-c" -a "$1" != "-d" ]; then
  usage
fi

if [ "$1" = "-c" ]; then
  if [ ! -d /RAMdisks ]; then
    mkdir /RAMdisks
  fi
  echo "Creating a $2 MB RAMdisk..."
  taille=`expr $2 \* 2048`
  id=`hdiutil attach -nomount ram://$taille`
  id2=`echo $id | cut -d "k" -f 2`
  if [ "$3" != "" ]; then
    label=$3
  else
    label="RAMdisk$id2"
  fi
  diskutil erasevolume HFS+ "$label" $id > /dev/null
  if [ $? = 1 ]; then
    echo "ERROR: Something went wrong while creating the RAMdisk."
    exit 1
  fi
  umount $id > /dev/null
  if [ $? = 1 ]; then
    echo "ERROR: Something went wrong while creating the RAMdisk."
    hdiutil detach $id > /dev/null
    exit 1
  fi
  hdiutil attach -mountpoint "/RAMdisks/${id2}_$label" $id > /dev/null
  if [ $? = 1 ]; then
    echo "ERROR: Something went wrong while creating the RAMdisk."
    hdiutil detach $id > /dev/null
    exit 1
  fi
  echo "RAMdisk $label created successfully. Its ID is $id2."
  exit 0
else
  if [ "$2" = "all" ]; then
    mount | grep /RAMdisks/ > /dev/null
    if [ $? = 1 ]; then
      echo "There is no RAMdisk to delete."
      exit 0
    fi
    echo "Deleting all RAMdisks..."
    for disk in `mount | grep /RAMdisks/ | cut -d " " -f 1`; do
      echo "  Deleting RAMdisk $disk..."
      hdiutil detach $disk > /dev/null
      if [ $? = 1 ]; then
        echo "ERROR: Something went wrong while deleting $disk."
        exit 1
      fi
      echo "  RAMdisk $disk deleted successfully."
    done
    echo "All RAMdisks deleted successfully."
    exit 0
  else
    found=0
    mount | grep "/RAMdisks/$2" > /dev/null
    if [ $? = 0 ]; then
      found=1
      dev="/dev/disk$2"
      name="/dev/disk$2"
    else
      mount | grep "$2" | grep /RAMdisks/ > /dev/null
      if [ $? = 0 ]; then
        count=`mount | grep "$2" | grep /RAMdisks | wc -l`
        count=`echo $count`
        if [ "$count" != "1" ]; then
          echo "ERROR: There are $count RAMdisks named $2. Please use ID instead of label."
          exit 1
        fi 
        found=1
        dev=`mount | grep "$2" | grep /RAMdisks | cut -d " " -f 1`
        name=$2
      fi
    fi
    if [ $found = 1 ]; then
      echo "Deleting RAMdisk $name..."
      hdiutil detach $dev > /dev/null
      if [ $? = 1 ]; then
        echo "ERROR: Something went wrong while deleting RAMdisk $name."
        exit 1
      fi
      echo "RAMdisk $name deleted successfully."
      exit 0
    else
      echo "ERROR: $2 is not a RAMdisk."
      exit 1
    fi
  fi 
fi
